Package: ggdendro
Version: 0.2.0
Title: Create Dendrograms and Tree Diagrams Using 'ggplot2'
Description: This is a set of tools for dendrograms and
    tree plots using 'ggplot2'.  The 'ggplot2' philosophy is to
    clearly separate data from the presentation.
    Unfortunately the plot method for dendrograms plots
    directly to a plot device without exposing the data.
    The 'ggdendro' package resolves this by making available
    functions that extract the dendrogram plot data. The package
    provides implementations for 'tree', 'rpart', as well as diana and agnes
    (from 'cluster') diagrams.
Authors@R: c(person("Andrie", "de Vries", role=c("aut", "cre"),
    email="apdevries@gmail.com"), person(c("Brian", "D."), "Ripley", role =
    c("aut"), comment = "author of package tree"))
License: GPL-2 | GPL-3
LazyLoad: true
ByteCompile: true
URL: https://andrie.github.io/ggdendro/
BugReports: https://github.com/andrie/ggdendro/issues
Depends: R(>= 3.5)
Imports: MASS, ggplot2(>= 0.9.2)
Suggests: rpart (>= 4.0-0), tree, testthat, knitr, cluster, scales,
        spelling, rmarkdown, covr
VignetteBuilder: knitr
RoxygenNote: 7.3.1
Language: en-US
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-02-18 21:50:45 UTC; apdev
Author: Andrie de Vries [aut, cre],
  Brian D. Ripley [aut] (author of package tree)
Maintainer: Andrie de Vries <apdevries@gmail.com>
Repository: CRAN
Date/Publication: 2024-02-23 12:40:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(dendro_data,default)
S3method(dendro_data,dendrogram)
S3method(dendro_data,hclust)
S3method(dendro_data,rpart)
S3method(dendro_data,tree)
S3method(dendro_data,twins)
export(as.dendro)
export(dendro_data)
export(dendro_data.default)
export(ggdendrogram)
export(is.dendro)
export(label)
export(leaf_label)
export(segment)
export(theme_dendro)
import(MASS)
import(ggplot2)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.new)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(stats,as.dendrogram)
importFrom(stats,complete.cases)
importFrom(stats,is.leaf)
importFrom(utils,head)
importFrom(utils,stack)
importFrom(utils,tail)

if (require(cluster)) {
  model <- agnes(votes.repub, metric = "manhattan", stand = TRUE)
  dg <- as.dendrogram(model)
  ggdendrogram(dg)
}


if (require(cluster)) {
  data(votes.repub)
  model <- diana(votes.repub, metric = "manhattan", stand = TRUE)
  dg <- as.dendrogram(model)
  ggdendrogram(dg)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----init---------------------------------------------------------------------
library(ggplot2)
library(ggdendro)

## ----dendrogram---------------------------------------------------------------
hc <- hclust(dist(USArrests), "ave")
ggdendrogram(hc, rotate = FALSE, size = 2)

## ----dendro1------------------------------------------------------------------
model <- hclust(dist(USArrests), "ave")
dhc <- as.dendrogram(model)
# Rectangular lines
ddata <- dendro_data(dhc, type = "rectangle")
p <- ggplot(segment(ddata)) + 
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) + 
  coord_flip() + 
  scale_y_reverse(expand = c(0.2, 0))
p

## ----dendro-2-----------------------------------------------------------------
p + 
  coord_flip() + 
  theme_dendro()

## ----dendro-3-----------------------------------------------------------------
ddata <- dendro_data(dhc, type = "triangle")
ggplot(segment(ddata)) + 
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) + 
  coord_flip() + 
  scale_y_reverse(expand = c(0.2, 0)) +
  theme_dendro()

## ----tree---------------------------------------------------------------------
if(require(tree)){
  data(cpus, package = "MASS")
  model <- tree(log10(perf) ~ syct + mmin + mmax + cach + chmin + chmax, 
                data = cpus)
  tree_data <- dendro_data(model)
  ggplot(segment(tree_data)) +
    geom_segment(aes(x = x, y = y, xend = xend, yend = yend, size = n), 
                 colour = "blue", alpha = 0.5) +
    scale_size("n") +
    geom_text(data = label(tree_data), 
              aes(x = x, y = y, label = label), vjust = -0.5, size = 3) +
    geom_text(data = leaf_label(tree_data), 
              aes(x = x, y = y, label = label), vjust = 0.5, size = 2) +
    theme_dendro()
}

## ----rpart--------------------------------------------------------------------
if(require(rpart)){
  model <- rpart(Kyphosis ~ Age + Number + Start, 
                 method = "class", data = kyphosis)
  ddata <- dendro_data(model)
  ggplot() + 
    geom_segment(data = ddata$segments, 
                 aes(x = x, y = y, xend = xend, yend = yend)) + 
    geom_text(data = ddata$labels, 
              aes(x = x, y = y, label = label), size = 3, vjust = 0) +
    geom_text(data = ddata$leaf_labels, 
              aes(x = x, y = y, label = label), size = 3, vjust = 1) +
    theme_dendro()
}

## ----twins--------------------------------------------------------------------
if(require(cluster)){
  model <- agnes(votes.repub, metric = "manhattan", stand = TRUE)
  dg <- as.dendrogram(model)
  ggdendrogram(dg)
  
  model <- diana(votes.repub, metric = "manhattan", stand = TRUE)
  dg <- as.dendrogram(model)
  ggdendrogram(dg)
}

require(ggplot2)

### Demonstrate dendro_data.dendrogram

model <- hclust(dist(USArrests), "ave")
dendro <- as.dendrogram(model)

# Rectangular lines
ddata <- dendro_data(dendro, type = "rectangle")
ggplot(segment(ddata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  coord_flip() +
  scale_y_reverse(expand = c(0.2, 0)) +
  theme_dendro()

# Triangular lines
ddata <- dendro_data(dendro, type = "triangle")
ggplot(segment(ddata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  theme_dendro()

# Demonstrate dendro_data.hclust

require(ggplot2)
hc <- hclust(dist(USArrests), "ave")

# Rectangular lines
hcdata <- dendro_data(hc, type = "rectangle")
ggplot(segment(hcdata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  coord_flip() +
  scale_y_reverse(expand = c(0.2, 0)) +
  theme_dendro()

# Triangular lines
hcdata <- dendro_data(hc, type = "triangle")
ggplot(segment(hcdata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  theme_dendro()

### Demonstrate rpart

if (require(rpart)) {
  require(ggplot2)
  fit <- rpart(Kyphosis ~ Age + Number + Start, method = "class", 
               data = kyphosis)
  fitr <- dendro_data(fit)
  ggplot() +
    geom_segment(data = fitr$segments, 
                 aes(x = x, y = y, xend = xend, yend = yend)
    ) +
    geom_text(data = fitr$labels, aes(x = x, y = y, label = label)) +
    geom_text(data = fitr$leaf_labels, aes(x = x, y = y, label = label)) +
    theme_dendro()
}

### Demonstrate ggdendrogram

library(ggplot2)
hc <- hclust(dist(USArrests), "ave")

# Demonstrate plotting directly from object class hclust
p <- ggdendrogram(hc, rotate = FALSE)
print(p)
ggdendrogram(hc, rotate = TRUE)

# demonstrate converting hclust to dendro using dendro_data first
hcdata <- dendro_data(hc)
ggdendrogram(hcdata, rotate = TRUE, size = 2) + 
  labs(title = "Dendrogram in ggplot2")

### Demonstrate tree

if (require(tree)) {
  require(ggplot2)
  require(MASS)
  data(cpus, package = "MASS")
  cpus.ltr <- tree(log10(perf) ~ syct + mmin + mmax + cach + chmin + chmax, 
                   data = cpus)
  tree_data <- dendro_data(cpus.ltr)
  ggplot(segment(tree_data)) +
    geom_segment(aes(x = x, y = y, xend = xend, yend = yend, linewidth = n),
      colour = "lightblue"
    ) +
    scale_size("n") +
    geom_text(
      data = label(tree_data),
      aes(x = x, y = y, label = label), vjust = -0.5, size = 4
    ) +
    geom_text(
      data = leaf_label(tree_data),
      aes(x = x, y = y, label = label), vjust = 0.5, size = 3
    ) +
    theme_dendro()
}

# modified version of stats:::plotNode
plotNode <-  function (x1, x2, subtree, type, center, leaflab, dLeaf, nodePar, 
            edgePar, horiz = FALSE) 
  {
    ddsegments <- NULL
    ddlabels <- list()
    
    wholetree <- subtree
    depth <- 0L
    llimit <- list()
    KK <- integer()
    kk <- integer()
    repeat {
      inner <- !is.leaf(subtree) && x1 != x2
      yTop <- attr(subtree, "height")
      bx <- plotNodeLimit(x1, x2, subtree, center)
      xTop <- bx$x
      depth <- depth + 1L
      llimit[[depth]] <- bx$limit
      hasP <- !is.null(nPar <- attr(subtree, "nodePar"))
      if (!hasP) 
        nPar <- nodePar
      if (getOption("verbose")) {
        cat(if (inner) 
          "inner node"
          else "leaf", ":")
        if (!is.null(nPar)) {
          cat(" with node pars\n")
          utils::str(nPar)
        }
        cat(if (inner) 
          paste(" height", formatC(yTop), "; "), "(x1,x2)= (", 
          formatC(x1, width = 4), ",", formatC(x2, width = 4), 
          ")", "--> xTop=", formatC(xTop, width = 8), "\n", 
          sep = "")
      }
      Xtract <- function(nam, L, default, indx) 
        rep(if (nam %in%  names(L)) L[[nam]] else default, length.out = indx)[indx]
      asTxt <- function(x) if (is.character(x) || is.expression(x) ||  is.null(x))  x else as.character(x)
      i <- if (inner || hasP)  1 else 2
      if (!is.null(nPar)) {
        pch <- Xtract("pch", nPar, default = 1L:2, i)
        cex <- Xtract("cex", nPar, default = c(1, 1), i)
        col <- Xtract("col", nPar, default = par("col"), 
                      i)
        bg <- Xtract("bg", nPar, default = par("bg"), i)
        points(if (horiz) 
          cbind(yTop, xTop)
          else cbind(xTop, yTop), pch = pch, bg = bg, col = col, 
          cex = cex)
      }
      if (leaflab == "textlike") 
        p.col <- Xtract("p.col", nPar, default = "white",  i)
      lab.col <- Xtract("lab.col", nPar, default = par("col"),  i)
      lab.cex <- Xtract("lab.cex", nPar, default = c(1, 1),  i)
      lab.font <- Xtract("lab.font", nPar, default = par("font"),  i)
      lab.xpd <- Xtract("xpd", nPar, default = c(TRUE, TRUE),  i)
      if (is.leaf(subtree)) {
        if (leaflab == "perpendicular") {
          if (horiz) {
            X <- yTop + dLeaf * lab.cex
            Y <- xTop
            srt <- 0
            adj <- c(0, 0.5)
          }
          else {
            Y <- yTop - dLeaf * lab.cex
            X <- xTop
            srt <- 90
            adj <- 1
          }
          nodeText <- asTxt(attr(subtree, "label"))
          # text(X, Y, nodeText, xpd = lab.xpd, srt = srt, 
          #      adj = adj, cex = lab.cex, col = lab.col, font = lab.font)
          ddlabels$xy <- c(ddlabels$xy, X, 0)
          ddlabels$text <- c(ddlabels$text, nodeText)
        }
      }
      else if (inner) {
        # segmentsHV <- function(x0, y0, x1, y1) {
        #   if (horiz) 
        #     segments(y0, x0, y1, x1, col = col, lty = lty,  lwd = lwd)
        #   else segments(x0, y0, x1, y1, col = col, lty = lty,  lwd = lwd)
        # }
        for (k in seq_along(subtree)) {
          child <- subtree[[k]]
          yBot <- attr(child, "height")
          if (getOption("verbose")) 
            cat("ch.", k, "@ h=", yBot, "; ")
          if (is.null(yBot)) 
            yBot <- 0
          xBot <- if (center) 
            mean(bx$limit[k:(k + 1)])
          else bx$limit[k] + .midDend(child)
          hasE <- !is.null(ePar <- attr(child, "edgePar"))
          if (!hasE) 
            ePar <- edgePar
          i <- if (!is.leaf(child) || hasE)  1 else 2
          col <- Xtract("col", ePar, default = par("col"),  i)
          lty <- Xtract("lty", ePar, default = par("lty"),  i)
          lwd <- Xtract("lwd", ePar, default = par("lwd"),  i)
          if (type == "triangle") {
            # segmentsHV(xTop, yTop, xBot, yBot)
            # segmentsHV(xTop, yTop, xBot, yBot)
            # ddsegments <- rbind(ddsegments, data.frame(x = xTop, y = yTop, xend = xBot, yend = yBot))
            ddsegments <- c(ddsegments, xTop, yTop, xBot, yBot)
          }
          else {
            # segmentsHV(xTop, yTop, xBot, yTop)
            # segmentsHV(xBot, yTop, xBot, yBot)
            # ddsegments <- rbind(ddsegments, data.frame(x = xTop, y = yTop, xend = xBot, yend = yTop))
            # ddsegments <- rbind(ddsegments, data.frame(x = xBot, y = yTop, xend = xBot, yend = yBot))
            ddsegments <- c(ddsegments, xTop, yTop, xBot, yTop)
            ddsegments <- c(ddsegments, xBot, yTop, xBot, yBot)
          }
          vln <- NULL
          # if (is.leaf(child) && leaflab == "textlike") {
          #   nodeText <- asTxt(attr(child, "label"))
          #   if (getOption("verbose"))
          #     cat("-- with \"label\"", format(nodeText))
          #   hln <- 0.6 * strwidth(nodeText, cex = lab.cex)/2
          #   vln <- 1.5 * strheight(nodeText, cex = lab.cex)/2
          #   rect(xBot - hln, yBot, xBot + hln, yBot + 2 *
          #          vln, col = p.col)
          #   text(xBot, yBot + vln, nodeText, xpd = lab.xpd,
          #        cex = lab.cex, col = lab.col, font = lab.font)
          # }
          # if (!is.null(attr(child, "edgetext"))) {
          #   edgeText <- asTxt(attr(child, "edgetext"))
          #   if (getOption("verbose"))
          #     cat("-- with \"edgetext\"", format(edgeText))
          #   if (!is.null(vln)) {
          #     mx <- if (type == "triangle")
          #       (xTop + xBot + ((xTop - xBot)/(yTop - yBot)) *
          #          vln)/2
          #     else xBot
          #     my <- (yTop + yBot + 2 * vln)/2
          #   }
          #   else {
          #     mx <- if (type == "triangle")
          #       (xTop + xBot)/2
          #     else xBot
          #     my <- (yTop + yBot)/2
          #   }
          #   p.col <- Xtract("p.col", ePar, default = "white",  i)
          #   p.border <- Xtract("p.border", ePar, default = par("fg"),  i)
          #   p.lwd <- Xtract("p.lwd", ePar, default = lwd,  i)
          #   p.lty <- Xtract("p.lty", ePar, default = lty,  i)
          #   t.col <- Xtract("t.col", ePar, default = col,  i)
          #   t.cex <- Xtract("t.cex", ePar, default = 1,  i)
          #   t.font <- Xtract("t.font", ePar, default = par("font"),  i)
          #   vlm <- strheight(c(edgeText, "h"), cex = t.cex)/2
          #   hlm <- strwidth(c(edgeText, "m"), cex = t.cex)/2
          #   hl3 <- c(hlm[1L], hlm[1L] + hlm[2L], hlm[1L])
          #   if (horiz) {
          #     polygon(my + c(-hl3, hl3), mx + sum(vlm) *
          #               c(-1L:1L, 1L:-1L), col = p.col, border = p.border,
          #             lty = p.lty, lwd = p.lwd)
          #     text(my, mx, edgeText, cex = t.cex, col = t.col,
          #          font = t.font)
          #   }
          #   else {
          #     polygon(mx + c(-hl3, hl3), my + sum(vlm) *
          #               c(-1L:1L, 1L:-1L), col = p.col, border = p.border,
          #             lty = p.lty, lwd = p.lwd)
          #     text(mx, my, edgeText, cex = t.cex, col = t.col,
          #          font = t.font)
          #   }
          # }
        }
      }
      if (inner && length(subtree)) {
        KK[depth] <- length(subtree)
        if (storage.mode(kk) != storage.mode(KK)) 
          storage.mode(kk) <- storage.mode(KK)
        kk[depth] <- 1L
        x1 <- bx$limit[1L]
        x2 <- bx$limit[2L]
        subtree <- subtree[[1L]]
      }
      else {
        repeat {
          depth <- depth - 1L
          if (!depth || kk[depth] < KK[depth]) 
            break
        }
        if (!depth) 
          break
        length(kk) <- depth
        kk[depth] <- k <- kk[depth] + 1L
        x1 <- llimit[[depth]][k]
        x2 <- llimit[[depth]][k + 1L]
        subtree <- wholetree[[kk]]
      }
    }
    # invisible()
    list(
      segments = ddsegments,
      labels = ddlabels
    )
  }

#
#  ggdendro/R/dendro_tree.R by Andrie de Vries  Copyright (C) 2011-2015
#  Contains code adapted from stats/hclust.R Copyright (C) 1995-2015 The R Core Team
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
#




#' Extract line segment and label data from dendrogram or hclust object.
#'
#' Extract line segment and label data from [stats::dendrogram()] or
#' [stats::hclust()] object.  The resulting object is a list of data frames
#' containing line segment data and label data.
#'
#' @param model object of class "dendrogram", e.g. the output of as.dendrogram()
#'
#' @param type The type of plot, indicating the shape of the dendrogram.
#'   "rectangle" will draw rectangular lines, while "triangle" will draw
#'   triangular lines.
#' 
#' @param ... ignored
#' 
#' 
#' @export
#' @return
#' A list with components:
#' \item{segments}{Line segment data}
#' \item{labels}{Label data}
#'
#' @seealso [ggdendrogram()]
#' @rdname dendro_data
#' @family dendro_data methods
#' @family dendrogram/hclust functions
#' @example inst/examples/example_dendro_data.R
#'
dendro_data.dendrogram <- function(model, type = c("rectangle", "triangle"), ...) {
  hcdata <- dendrogram_data(model, type = type, ...)
  as.dendro(
    segments = hcdata$segments,
    labels = hcdata$labels,
    class = "dendrogram"
  )
}

#' @rdname dendro_data
#' @export
dendro_data.hclust <- function(model, type = c("rectangle", "triangle"), ...) {
  dhc <- as.dendrogram(model)
  hcdata <- dendrogram_data(dhc, type = type, ...)
  as.dendro(
    segments = hcdata$segments,
    labels = hcdata$labels,
    class = "hclust"
  )
}

#' @rdname dendro_data
#' @example inst/examples/example_dendro_twins.R
#' @export
dendro_data.twins <- function(model, type = c("rectangle", "triangle"), ...) {
  dhc <- as.dendrogram(model)
  hcdata <- dendrogram_data(dhc, type = type, ...)
  as.dendro(
    segments = hcdata$segments,
    labels = hcdata$labels,
    class = "hclust"
  )
}





#' Extract data frame from dendrogram object for plotting using ggplot.
#'
#' Extract data frame from dendrogram object for plotting using ggplot
#'
#' @param x object of class "dendrogram", e.g. the output of as.dendrogram()
#' 
#' @param type The type of plot, indicating the shape of the dendrogram:
#'   "rectangle" will draw rectangular lines, while "triangle" will draw
#'   triangular lines.
#' 
#' @param ... ignored
#' @seealso [ggdendrogram()]
#' @family dendro_data methods
#' @family dendrogram/hclust functions
#' @keywords internal
dendrogram_data <- function(x, type = c("rectangle", "triangle"), ...) {

  # Initialise variables that used to be in parameter list
  leaflab <- "perpendicular"
  center <- FALSE
  xlab <- ""
  ylab <- ""
  horiz <- FALSE
  # frame.plot <- FALSE
  xaxt <- "n"
  yaxt <- "s"
  nodePar <- NULL
  edgePar <- list()
  dLeaf <- NULL
  edge.root <- is.leaf(x) || !is.null(attr(x, "edgetext"))

  type <- match.arg(type)
  # leaflab <- match.arg(leaflab)
  hgt <- attr(x, "height")
  if (edge.root && is.logical(edge.root)) {
    edge.root <- 0.0625 * if (is.leaf(x)) 1 else hgt
  }
  mem.x <- .memberDend(x)
  yTop <- hgt + edge.root
  if (center) {
    x1 <- 0.5
    x2 <- mem.x + 0.5
  }
  else {
    x1 <- 1
    x2 <- mem.x
  }
  xl. <- c(x1 - 1 / 2, x2 + 1 / 2)
  yl. <- c(0, yTop)
  # 	if (missing(xlim) || is.null(xlim)) xlim <- xl.
  # 	if (missing(ylim) || is.null(ylim)) ylim <- yl.
  # 	plot(0, xlim = xlim, ylim = ylim, type = "n", xlab = xlab,
  # 			ylab = ylab, xaxt = xaxt, yaxt = yaxt, frame.plot = frame.plot,
  # 			...)
  # 	if (is.null(dLeaf))
  # 		dLeaf <- 0.75 * (if (horiz)
  # 						strwidth("w")
  # 					else strheight("x"))
  if (edge.root) {
    # 		x0 <- stats:::plotNodeLimit(x1, x2, x, center)$x
    # 		(x0, hgt, x0, yTop)
    if (!is.null(et <- attr(x, "edgetext"))) {
      my <- mean(hgt, yTop)
      # 			text(x0, my, et)
    }
  }
  ret <- plotNode(x1, x2, x,
    type = type, center = center, leaflab = leaflab,
    dLeaf = dLeaf, nodePar = nodePar, edgePar = edgePar, horiz = FALSE
  )
  ret$segments <- as.data.frame(matrix(
    ret$segments, ncol = 4, byrow = TRUE, 
    dimnames = list(NULL, c("x", "y", "xend", "yend"))
    ))
  
  ret$labels <- cbind(
    as.data.frame(matrix(ret$labels$xy, ncol = 2, byrow = TRUE, dimnames = list(NULL, c("x", "y")))),
    data.frame(label = ret$labels$text)
  )
  
  ret
}

#
#  ggdendro/R/dendro_tree.R by Andrie de Vries  Copyright (C) 2011-2015
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
#



#  ------------------------------------------------------------------------



#' Create Dendrograms and Tree Diagrams using 'ggplot2'
#'
#' This package enables you to create dendrograms and tree plots using
#' [ggplot2::ggplot()].
#'
#' The `ggplot2` philosophy is to clearly separate data from the presentation.
#' Unfortunately the plot method for dendrograms ([plot.dendrogram()]) plots
#' directly to a plot device without exposing the data.  The `ggdendro` package
#' resolves this by making available functions that extract the dendrogram plot
#' data.  This data can be used with `ggplot`.
#'
#' The function [dendro_data()] extracts data from different objects that
#' contain dendrogram information.  It is a generic function with methods for:
#'
#' - `hclust`: [dendro_data.hclust()]
#' - dendrogram: [dendro_data.dendrogram()]
#' - regression trees: [dendro_data.tree()]
#' - partition trees: [dendro_data.rpart()]
#' - `agnes` and `diana`: [dendro_data.twins()]
#'
#' These methods create an object of class `dendro`, consisting of a list of
#' data frames.  To extract the relevant data frames from the list, you can use
#' the accessor functions:
#'
#' - [segment()]: the line segment data
#' - [label()]: the text for each end segment
#' - [leaf_label()]: the leaf labels of a tree diagram
#'
#' To plot a dendrogram, either construct a plot with [ggplot2::ggplot()] or use
#' the function [ggdendrogram()].
#' 
#' @import MASS ggplot2
#' @importFrom grDevices col2rgb dev.cur dev.new
#' @importFrom graphics par points polygon text
#' @importFrom stats as.dendrogram complete.cases is.leaf
#' @importFrom utils head stack tail
#' @author Andrie de Vries - \email{apdevries@@gmail.com}
#' @keywords package
#' @seealso [dendro_data()]
"_PACKAGE"

#
#  ggdendro/R/dendro_tree.R by Andrie de Vries  Copyright (C) 2011-2015
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
#


#' Extract cluster data from a model into a list of data frames.
#'
#' This function provides a generic mechanism to extract relevant plotting data,
#' typically line segments and labels, from a variety of cluster models.
#'
#' For [stats::dendrogram()] and `tree::tree()` models, extracts line segment
#' data and labels.
#'
#'
#' @param model object of type [stats::hclust()], [stats::dendrogram()] or
#'   [tree::tree()]
#'
#' @param ... ignored
#'
#' @export dendro_data dendro_data.default
#'
#' @return a list of data frames that contain the data appropriate to each
#'   cluster model
#'
#' @seealso There are several implementations for specific cluster algorithms:
#'
#' - [dendro_data.hclust()] 
#' - [dendro_data.dendrogram()]
#' - [dendro_data.tree()]
#' - [dendro_data.rpart()]
#' 
#'  To extract the data for line segments, labels or leaf labels use:
#'  
#' - [segment()]: the line segment data
#' - [label()]: the text for each end segment
#' - [leaf_label()]: the leaf labels of a tree diagram
#' 
dendro_data <- function(model, ...) {
  UseMethod("dendro_data", model)
}

#' @rdname dendro_data
#' @export
dendro_data.default <- function(model, ...) {
  x <- class(model)
  stop(paste("No dendro_data method defined for class", x))
  return(NULL)
}

#' Tests whether an object is of class dendro.
#'
#' Is a dendro?  Tests whether an object is of class dendro.
#'
#' @param x Object to check
#' @export
#' @seealso [dendro_data()] and [ggdendro-package()]
is.dendro <- function(x) {
  inherits(x, "dendro")
}

#' Coerces object to class dendro.
#'
#' Method for coercing object to class dendro.
#'
#' @param segments data.frame with segment data
#'
#' @param labels data.frame with labels data
#'
#' @param leaf_labels data.frame with leaf label data
#'
#' @param class The class of the original model object, e.g. "hclust".  This is
#'   used by [ggdendrogram()] to determine the angle and justification
#'   of labels
#'
#' @seealso [dendro_data()] and [ggdendro-package()]
#'
#' @export
as.dendro <- function(segments, labels, leaf_labels = NULL, class) {
  if (missing(class)) stop("Missing class in as.dendro")
  x <- list(
    segments = segments,
    labels = labels,
    leaf_labels = leaf_labels,
    class = class
  )
  class(x) <- "dendro"
  x
}


#' Returns segment, label or leaf-label data from dendro object.
#'
#' `segment` extracts line segments, `label` extracts labels, and `leaf_label`
#' extracts leaf labels from a dendro object.
#'
#' @param x dendro object
#' @aliases segment label leaf_label
#' @seealso [dendro_data()]
#' @export
segment <- function(x) {
  x$segments
}

#' @rdname segment
#' @export
label <- function(x) {
  x$labels
}

#' @rdname segment
#' @export
leaf_label <- function(x) {
  x$leaf_labels
}

context("clusterplots and dendrograms")

test_that("dendrogram plots", {
  
  require(ggplot2)  
  hc <- hclust(dist(USArrests), "ave")
  hcdata <- dendro_data(hc, type="rectangle")
  p <- ggplot() + 
    geom_segment(data=segment(hcdata), aes(x=x0, y=y0, xend=x1, yend=y1)) +
    geom_text(data=label(hcdata), aes(x=x, y=y, label=text)) +
    coord_flip() + scale_y_reverse(expand=c(0.2, 0))
  expect_that(p, is_a("ggplot"))
  #print(p)
  
})

# Test functionality using testthat library
# 
# Author: Andrie
#------------------------------------------------------------------------------

# tree

# data(iris)
# iris <- iris[, -5]

#------------------------------------------------------------------------------

context("dendrogram")

test_that("data_dendrogram() returns a correct classes", {

			hc <- hclust(dist(USArrests), "ave")
			dhc <- as.dendrogram(hc)
			
			expect_is(dendro_data(dhc, type="rectangle"), "dendro")

			ddata <- dendro_data(dhc, type="rectangle")
			expect_that(ddata$segments, is_a("data.frame"))
			expect_that(ddata$labels, is_a("data.frame"))
			expect_that(nrow(ddata$segments), equals(196))
			
			ddata <- dendro_data(dhc, type="triangle")
			expect_that(ddata$segments, is_a("data.frame"))
			expect_that(ddata$labels, is_a("data.frame"))
			expect_that(nrow(ddata$segments), equals(98))
			
		})

#------------------------------------------------------------------------------

test_that("undefined model type throws error", {
  expect_that(dendro_data(USArrests), throws_error())
})

context("rpart")

test_that("data_tree() returns the correct classes", {
  
  if(!require("rpart", quietly = TRUE)) skip("package rpart not available")
  
  fit <- rpart(Kyphosis ~ Age + Number + Start,   method="class", data=kyphosis)
  tdata <- dendro_data(fit)
  expect_is(tdata, "dendro")
  expect_is(segment(tdata), "data.frame")
  expect_is(label(tdata), "data.frame")
  expect_is(leaf_label(tdata), "data.frame")
  
})

context("hclust")

test_that("data_hclust() returns the correct classes", {
  
  hc <- hclust(dist(USArrests), "ave")
  
  expect_is(dendro_data(hc, type="rectangle"), "dendro")
  
  ddata <- dendro_data(hc, type="rectangle")
  expect_is(segment(ddata), "data.frame")
  expect_is(label(ddata), "data.frame")
  expect_equal(nrow(segment(ddata)), 196)
  
})

context("tree")

test_that("data_tree() returns the correct classes", {
  if(!require("tree", quietly = TRUE)) skip("package tree not available")
  
  data(cpus, package="MASS")
  cpus.ltr <- tree(log10(perf) ~ syct + mmin + mmax + cach + chmin + chmax, data = cpus)

  expect_that(dendro_data(cpus.ltr), is_a("dendro"))
  tdata <- dendro_data(cpus.ltr)
  expect_that(segment(tdata), is_a("data.frame"))
  expect_that(label(tdata), is_a("data.frame"))
  expect_that(leaf_label(tdata), is_a("data.frame"))
  
})

#
#  ggdendro/R/dendro_rpart.R by Andrie de Vries  Copyright (C) 2011-2015
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
# Classification Tree with rpart


#' Extract data from classification tree object for plotting using ggplot.
#'
#' Extracts data to plot line segments and labels from a
#' [rpart::rpart()] classification tree object.  This data can then be
#' manipulated or plotted, e.g. using [ggplot2::ggplot()].
#'
#' This code is in essence a copy of [rpart::plot.rpart()], retaining
#' the plot data but without plotting to a plot device.
#'
#' @param model object of class "tree", e.g. the output of tree()
#'
#' @param uniform if TRUE, uniform vertical spacing of the nodes is used; this
#'   may be less cluttered when fitting a large plot onto a page. The default is
#'   to use a non-uniform spacing proportional to the error in the fit.
#'
#' @param branch controls the shape of the branches from parent to child node.
#'   Any number from 0 to 1 is allowed. A value of 1 gives square shouldered
#'   branches, a value of 0 give V shaped branches, with other values being
#'   intermediate.
#'
#' @param compress if FALSE, the leaf nodes will be at the horizontal plot
#'   coordinates of 1:nleaves. If TRUE, the routine attempts a more compact
#'   arrangement of the tree. The compaction algorithm assumes uniform=TRUE;
#'   surprisingly, the result is usually an improvement even when that is not
#'   the case.
#'
#' @param nspace the amount of extra space between a node with children and a
#'   leaf, as compared to the minimal space between leaves. Applies to
#'   compressed trees only. The default is the value of branch.
#'
#' @param minbranch	set the minimum length for a branch to minbranch times the
#'   average branch length. This parameter is ignored if uniform=TRUE. Sometimes
#'   a split will give very little improvement, or even (in the classification
#'   case) no improvement at all. A tree with branch lengths strictly
#'   proportional to improvement leaves no room to squeeze in node labels.
#'
#' @param ... ignored
#' @export
#' @return
#' A list of three data frames:
#' \item{segments}{a data frame containing the line segment data}
#' \item{labels}{a data frame containing the label text data}
#' \item{leaf_labels}{a data frame containing the leaf label text data}
#'
#' @seealso [ggdendrogram()]
#' @family dendro_data methods
#' @family rpart functions
#' @example inst/examples/example_dendro_rpart.R
#'
dendro_data.rpart <- function(model, uniform = FALSE, branch = 1, compress = FALSE,
                              nspace, minbranch = 0.3, ...) {
  x <- model
  if (!inherits(x, "rpart")) stop("Not a legitimate \"rpart\" object")
  if (nrow(x$frame) <= 1L) stop("fit is not a tree, just a root")

  if (compress & missing(nspace)) nspace <- branch
  if (!compress) nspace <- -1L # means no compression
  # if(!interactive()) if (dev.cur() == 1L) dev.new() # not needed in R

  parms <- list(
    uniform = uniform,
    branch = branch,
    nspace = nspace,
    minbranch = minbranch
  )

  ## define the plot region
  temp <- rpartco(x, parms = parms)
  xx <- temp$x
  yy <- temp$y
  # temp1 <- range(xx) + diff(range(xx)) * c(-margin, margin)
  # temp2 <- range(yy) + diff(range(yy)) * c(-margin, margin)
  #   plot(temp1, temp2, type = "n", axes = FALSE, xlab = "", ylab = "", ...)
  ## Save information per device, once a new device is opened.
  # assign(paste0("device", dev.cur()), parms, envir = rpart_ggdendro_env)

  # Draw a series of horseshoes or V's, left son, up, down to right son
  #   NA's in the vector cause lines() to "lift the pen"
  node <- as.numeric(row.names(x$frame))
  temp <- rpart.branch(xx, yy, node, branch)

  #   if (branch > 0) text(xx[1L], yy[1L], "|")
  #   lines(c(temp$x), c(temp$y))
  #   invisible(list(x = xx, y = yy))

  labels <- text_rpart(x, parms = parms)

  segments <- rpart_segments(temp)
  #   labels <- rpart_labels(xx, ...)
  as.dendro(
    segments = segments,
    labels = labels$labels,
    leaf_labels = labels$leaf_labels,
    class = "rpart"
  )
}

#' Extract data frame from rpart object for plotting using ggplot.
#' @param ... ignored
#' @keywords internal
#' @seealso [ggdendrogram()]
#' @family rpart functions
rpart_segments <- function(x, ...) {
  dat <- data.frame(
    stack(as.data.frame(x$x)),
    stack(as.data.frame(x$y))
  )[, c("ind", "values", "values.1")]

  dat <- cbind(head(dat, -1), tail(dat, -1))
  dat <- dat[complete.cases(dat), -4]
  names(dat) <- c("n", "x", "y", "xend", "yend")
  dat
}


#' Extract labels data frame from rpart object for plotting using ggplot.
#'
#' This code is modified from the original plot.rpart in package rpart.
#' @return a list with two elements: $labels and $leaf_labels
#' @author Original author Brian Ripley
#' @keywords internal
#' @seealso [ggdendrogram()]
#' @family dendro_data methods
#' @family rpart functions
rpart_labels <- function(x) {
  labelSplits <- labelLeaves <- NULL
  list(
    labels = labelSplits,
    leaf_labels = labelLeaves
  )
}

#
#  ggdendro/R/dendro_tree.R by Andrie de Vries  Copyright (C) 2011-2015
#  Contains code from tree/R/tree.R by B. D. Ripley  Copyright (C) 1994-2015
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
#


# tree_ggdendro_env <- new.env()

#' Extract data from regression tree object for plotting using ggplot.
#'
#' Extracts data to plot line segments and labels from a `tree::tree()` object.
#' This data can then be manipulated or plotted, e.g. using [ggplot2::ggplot()].
#'
#' @param model object of class "tree", e.g. the output of tree()
#'
#' @param type Either `proportional` or `uniform`. If this partially matches
#'   "uniform", the branches are of uniform length. Otherwise they are
#'   proportional to the decrease in impurity.
#'
#' @param ... ignored
#'
#' @export
#'
#' @return
#' A list of three data frames:
#' \item{segments}{a data frame containing the line segment data}
#' \item{labels}{a data frame containing the label text data}
#' \item{leaf_labels}{a data frame containing the leaf label text data}
#'
#' @seealso [ggdendrogram()]
#' @family dendro_data methods
#' @family tree functions
#' @author Andrie de Vries, using code modified from original by Brian Ripley
#' @example inst/examples/example_dendro_tree.R
dendro_data.tree <- function(model, type = c("proportional", "uniform"), ...) {
  type <- match.arg(type)
  uniform <- type == "uniform"

  # dev <- dev.cur()
  # if (dev == 1L) dev <- 2L # as device will be opened.

  # assign(paste0("device", dev), uniform, envir = tree_ggdendro_env)

  labels <- tree_labels(model, uniform = uniform, ...)
  as.dendro(
    segments = tree_segments(model, uniform, ...),
    labels = labels$labels,
    leaf_labels = labels$leaf_labels,
    class = "tree"
  )
}



#' Extract data frame from tree object for plotting using ggplot.
#' @param model object of class "tree", e.g. the output of tree()
#' @param ... ignored
#' @keywords internal
#' @seealso [ggdendrogram()]
#' @family tree functions
#' @author Code modified from original by Brian Ripley
tree_segments <- function(model, uniform, ...) {
  if (missing(uniform)) stop("specify the uniform argument")
  # Uses tree:::treeco to extract data frame of plot locations
  xy <- treeco(model, uniform = uniform)
  n <- model$frame$n

  # Lines copied from tree:::treepl
  x <- xy$x
  y <- xy$y
  node <- as.numeric(row.names(model$frame))
  parent <- match((node %/% 2), node)
  sibling <- match(ifelse(node %% 2, node - 1L, node + 1L), node)

  linev <- data.frame(x = x, y = y, xend = x, yend = y[parent], n = n)
  lineh <- data.frame(x = x[parent], y = y[parent], xend = x, yend = y[parent], 
                      n = n)

  rbind(linev[-1, ], lineh[-1, ])
}

#' Extract labels data frame from tree object for plotting using ggplot.
#' @param model object of class "tree", e.g. the output of tree()
#' @param ... ignored
#' @return a list with two elements: $labels and $leaf_labels
#' @keywords internal
#' @seealso [ggdendrogram()]
#' @family tree functions
#' @author Code modified from original by Brian Ripley
tree_labels <- function(model, uniform, ...) {
  # Uses tree:::treeco to extract data frame of plot locations
  xy <- treeco(model, uniform = uniform)
  label <- model$frame$var
  yval <- model$frame$yval
  sleft <- model$frame$splits.cutleft
  sright <- model$frame$splits.right

  # Lines copied from tree:::treepl
  x <- xy$x
  y <- xy$y
  node <- as.numeric(row.names(model$frame))
  parent <- match((node %/% 2), node)
  sibling <- match(ifelse(node %% 2, node - 1L, node + 1L), node)

  # Extract labels
  data <- data.frame(x = x, y = y, label = label)
  data <- data[data$label != "<leaf>", ]
  labels <- as.data.frame(data)

  # Extract leaf labels
  data <- data.frame(x, y, label, yval)
  data <- data[data$label == "<leaf>", ]
  if (is.numeric(data$yval)) {
    data$label <- round(data$yval, 2)
  } else {
    data$label <- data$yval
  }
  leaf_labels <- as.data.frame(data)

  list(
    labels = labels,
    leaf_labels = leaf_labels
  )
}

#' Extract labels data frame from tree object for plotting using ggplot.
#'
#' Extract labels data frame from tree object for plotting using ggplot
#'
#' @param model object of class "tree", e.g. the output of tree()
#' @param ... ignored
#' @keywords internal
#' @seealso [ggdendrogram()]
#' @family tree functions
#' @author Code modified from original by Brian Ripley
get_data_tree_leaf_labels <- function(model, uniform, ...) {
  # Uses tree:::treeco to extract data frame of plot locations
  xy <- treeco(model, uniform = uniform)
  label <- model$frame$var
  yval <- model$frame$yval
  sleft <- model$frame$splits.cutleft
  sright <- model$frame$splits.right

  # Lines copied from tree:::treepl
  x <- xy$x
  y <- xy$y
  node <- as.numeric(row.names(model$frame))
  parent <- match((node %/% 2), node)
  sibling <- match(ifelse(node %% 2, node - 1L, node + 1L), node)

  data <- data.frame(x, y, label, yval)
  data <- data[data$label == "<leaf>", ]
  data$label <- round(data$yval, 2)
  data
}


# treeco ------------------------------------------------------------------

#' Function copied from tree:::treeco.
#'
#' @param tree tree object
#' @param uniform ???
#' @keywords internal
treeco <- function(tree, uniform) {
  # if (missing(uniform)) {
  #   pn <- paste0("device", dev.cur())
  #   uniform <- if (exists(pn, envir = tree_ggdendro_env, inherits = FALSE))
  #     get(pn, envir = tree_ggdendro_env, inherits = FALSE)
  #   else FALSE
  # }
  if (missing(uniform)) stop("specify uniform argument")

  frame <- tree$frame
  node <- as.integer(row.names(frame))
  depth <- tree.depth(node)
  x <- -depth
  if (uniform) {
    y <- x
  } else {
    y <- dev <- frame$dev
    depth <- split(seq(node), depth)
    parent <- match(node %/% 2L, node)
    sibling <- match(
      ifelse(node %% 2L, node - 1L, node + 1L),
      node
    )
    for (i in depth[-1L]) {
      y[i] <- y[parent[i]] - dev[parent[i]] +
        dev[i] + dev[sibling[i]]
    }
  }
  depth <- -x
  leaves <- frame$var == "<leaf>"
  x[leaves] <- seq(sum(leaves))
  depth <- split(seq(node)[!leaves], depth[!leaves])
  left.child <- match(node * 2L, node)
  right.child <- match(node * 2 + 1L, node)
  for (i in rev(depth)) x[i] <- 0.5 * (x[left.child[i]] + x[right.child[i]])
  list(x = x, y = y)
}

#
#  ggdendro/R/dendro_rpart.R by Andrie de Vries and Brian Ripley
#  Copyright (C) 2011-2015
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
#


# labels.rpart ----------------------------------------------------------------


#' Make the nice labels used by print and summary.
#'
#' @param digits obvious
#'
#' @param minlength 0 = don't abbrev factors; 1 = use single letters; 2+= the
#'   same arg as the "abbreviate" function
#'
#' @param collapse an oddly named argument: FALSE = return a matrix with two
#'   columns, containing the labels of the left and right descendants of each
#'   node; TRUE = return a vector of 1 column, with the label of the parent
#'
#' @param pretty: for historical compatibility: 0   -> minlength = 0; NULL ->
#'   minlength = 1; TRUE   -> minlength = 4
#'
#' @param ... = other args for abbreviate()
#'
#' @keywords internal
#' @noRd
labels.rpart <- function(object, digits = 4, minlength = 1L, pretty,
                         collapse = TRUE, ...) {
  if (missing(minlength) && !missing(pretty)) {
    minlength <- if (is.null(pretty)) {
      1L
    } else if (is.logical(pretty)) {
      if (pretty) 4L else 0L
    } else {
      0L
    }
  }

  ff <- object$frame
  n <- nrow(ff)
  if (n == 1L) {
    return("root")
  } # special case of no splits

  is.leaf <- (ff$var == "<leaf>")
  whichrow <- !is.leaf
  vnames <- ff$var[whichrow] # the variable names for the primary splits

  index <- cumsum(c(1, ff$ncompete + ff$nsurrogate + !is.leaf))
  irow <- index[c(whichrow, FALSE)] # we only care about the primary split
  ncat <- object$splits[irow, 2L]

  ## Now to work: first create labels for the left and right splits,
  ##  but not for leaves of course
  ##
  lsplit <- rsplit <- character(length(irow))

  if (any(ncat < 2L)) { # any continuous vars ?
    jrow <- irow[ncat < 2L]
    cutpoint <- formatg(object$splits[jrow, 4L], digits)
    temp1 <- (ifelse(ncat < 0, "< ", ">="))[ncat < 2L]
    temp2 <- (ifelse(ncat < 0, ">=", "< "))[ncat < 2L]
    lsplit[ncat < 2L] <- paste0(temp1, cutpoint)
    rsplit[ncat < 2L] <- paste0(temp2, cutpoint)
  }

  if (any(ncat > 1L)) { # any categorical variables ?
    xlevels <- attr(object, "xlevels")
    ##
    ## jrow will be the row numbers of factors within lsplit and rsplit
    ## crow the row number in "csplit"
    ## and cindex the index on the "xlevels" list
    ##
    jrow <- seq_along(ncat)[ncat > 1L]
    crow <- object$splits[irow[ncat > 1L], 4L] # row number in csplit
    cindex <- (match(vnames, names(xlevels)))[ncat > 1L]

    ## Now, abbreviate the levels
    if (minlength == 1L) {
      if (any(ncat > 52L)) {
        warning("more than 52 levels in a predicting factor, truncated for printout",
          domain = NA
        )
      }
      xlevels <- lapply(xlevels, function(z) c(letters, LETTERS)[pmin(seq_along(z), 52L)])
    } else if (minlength > 1L) {
      xlevels <- lapply(xlevels, abbreviate, minlength, ...)
    }

    ## Now tuck in the labels
    ## I'll let some other clever person vectorize this
    for (i in seq_along(jrow)) {
      j <- jrow[i]
      splits <- object$csplit[crow[i], ]
      ## splits will contain 1=left, 3=right, 2= neither
      cl <- if (minlength == 1L) "" else ","
      lsplit[j] <-
        paste((xlevels[[cindex[i]]])[splits == 1L], collapse = cl)
      rsplit[j] <-
        paste((xlevels[[cindex[i]]])[splits == 3L], collapse = cl)
    }
  }

  if (!collapse) { # called by no routines that I know of
    ltemp <- rtemp <- rep("<leaf>", n)
    ltemp[whichrow] <- lsplit
    rtemp[whichrow] <- rsplit
    return(cbind(ltemp, rtemp))
  }

  lsplit <- paste0(ifelse(ncat < 2L, "", "="), lsplit)
  rsplit <- paste0(ifelse(ncat < 2L, "", "="), rsplit)

  ## Now match them up to node numbers
  ##   The output will have one label per row of object$frame, each
  ##   corresponding the the line segement joining this node to its parent
  varname <- (as.character(vnames))
  node <- as.numeric(row.names(ff))
  parent <- match(node %/% 2L, node[whichrow])
  odd <- (as.logical(node %% 2L))

  labels <- character(n)
  labels[odd] <- paste0(varname[parent[odd]], rsplit[parent[odd]])
  labels[!odd] <- paste0(varname[parent[!odd]], lsplit[parent[!odd]])
  labels[1L] <- "root"
  labels
}


# rpart_ggdendro_env ------------------------------------------------------

# rpart_ggdendro_env <- new.env()


# rpart.branch ------------------------------------------------------------



##
## Compute the "branches" to be drawn for an rpart object
##
## most likely this could simply default to branch = 1
rpart.branch <- function(x, y, node, branch) {
  # if (missing(branch)) {
  #   pn <- paste0("device", dev.cur())
  #   if (!exists(pn, envir = rpart_ggdendro_env, inherits = FALSE))
  #     stop("no information available on parameters from previous call to plot()")
  #   parms <- get(pn, envir = rpart_ggdendro_env, inherits = FALSE)
  #   branch <- parms$branch
  # }
  if (missing(branch)) branch <- 1

  ## Draw a series of horseshoes, left son, up, over, down to right son
  ##   NA's in the vector cause lines() to "lift the pen"
  is.left <- (node %% 2L == 0L) # left hand sons
  node.left <- node[is.left]
  parent <- match(node.left / 2L, node)
  sibling <- match(node.left + 1L, node)
  temp <- (x[sibling] - x[is.left]) * (1 - branch) / 2
  xx <- rbind(
    x[is.left], x[is.left] + temp,
    x[sibling] - temp, x[sibling], NA
  )
  yy <- rbind(y[is.left], y[parent], y[parent], y[sibling], NA)
  list(x = xx, y = yy)
}


# tree.depth ------------------------------------------------------------------

tree.depth <- function(nodes) {
  depth <- floor(log(nodes, base = 2) + 1e-7)
  depth - min(depth)
}


# rpartco ---------------------------------------------------------------------

#' Compute the x-y coordinates for a tree.
#'
#' @param tree Tree model
#' @param parms Graphics parameters
#' @keywords internal
rpartco <- function(tree, parms) {
  # if (missing(parms)) {
  #   pn <- paste0("device", dev.cur())
  #   if (!exists(pn, envir = rpart_ggdendro_env, inherits = FALSE))
  #     stop("no information available on parameters from previous call to plot()")
  #   parms <- get(pn, envir = rpart_ggdendro_env, inherits = FALSE)
  # }

  frame <- tree$frame
  node <- as.numeric(row.names(frame))
  depth <- tree.depth(node)
  is.leaf <- (frame$var == "<leaf>")
  if (length(parms)) {
    uniform <- parms$uniform
    nspace <- parms$nspace
    minbranch <- parms$minbranch
  } else {
    uniform <- FALSE
    nspace <- -1
    minbranch <- 0.3
  }

  if (uniform) {
    y <- (1 + max(depth) - depth) / max(depth, 4L)
  } else { # make y- (parent y) = change in deviance
    y <- dev <- frame$dev
    temp <- split(seq(node), depth) # depth 0 nodes, then 1, then ...
    parent <- match(node %/% 2L, node)
    sibling <- match(ifelse(node %% 2L, node - 1L, node + 1L), node)

    ## assign the depths
    for (i in temp[-1L]) {
      temp2 <- dev[parent[i]] - (dev[i] + dev[sibling[i]])
      y[i] <- y[parent[i]] - temp2
    }
    ##
    ## For some problems, classification & loss matrices in particular
    ##   the gain from a split may be 0.  This is ugly on the plot.
    ## Hence the "fudge" factor of  0.3 * the average step
    ##
    fudge <- minbranch * diff(range(y)) / max(depth)
    for (i in temp[-1L]) {
      temp2 <- dev[parent[i]] - (dev[i] + dev[sibling[i]])
      haskids <- !(is.leaf[i] & is.leaf[sibling[i]])
      y[i] <- y[parent[i]] - ifelse(temp2 <= fudge & haskids, fudge, temp2)
    }
    y <- y / (max(y))
  }

  # Now compute the x coordinates, by spacing out the leaves and then
  #   filling in
  x <- double(length(node)) # allocate, then fill it in below
  x[is.leaf] <- seq(sum(is.leaf)) # leaves at 1, 2, 3, ....
  left.child <- match(node * 2L, node)
  right.child <- match(node * 2L + 1L, node)

  ## temp is a list of non-is.leaf, by depth
  temp <- split(seq(node)[!is.leaf], depth[!is.leaf])
  for (i in rev(temp)) {
    x[i] <- 0.5 * (x[left.child[i]] + x[right.child[i]])
  }

  if (nspace < 0) {
    return(list(x = x, y = y))
  }

  ##
  ## Now we get fancy, and try to do overlapping
  ##
  ##  The basic algorithm is, at each node:
  ##      1: get the left & right edges, by depth, for the left and
  ##           right sons, of the x-coordinate spacing.
  ##      2: find the minimal free spacing.  If this is >0, slide the
  ##           right hand son over to the left
  ##      3: report the left & right extents of the new tree up to the
  ##           parent
  ##   A way to visualize steps 1 and 2 is to imagine, for a given node,
  ##      that the left son, with all its descendants, is drawn on a
  ##      slab of wood.  The left & right edges, per level, give the
  ##      width of this board.  (The board is not a rectangle, it has
  ##      'stair step' edges). Do the same for the right son.  Now
  ##      insert some spacers, one per level, and slide right hand
  ##      board over until they touch.  Glue the boards and spacer
  ##      together at that point.
  ##
  ##  If a node has children, its 'space' is considered to extend left
  ##    and right by the amount "nspace", which accounts for space
  ##    used by the arcs from this node to its children.  For
  ##    horseshoe connections nspace usually is 1.
  ##
  compress <- function(x, me, depth) {
    lson <- me + 1L
    if (is.leaf[lson]) {
      left <- list(
        left = x[lson], right = x[lson],
        depth = depth + 1L, sons = lson
      )
    } else {
      left <- compress(x, me + 1L, depth + 1L)
      x <- left$x
    }

    rson <- me + 1L + length(left$sons) # index of right son
    if (is.leaf[rson]) {
      right <- list(
        left = x[rson], right = x[rson],
        depth = depth + 1L, sons = rson
      )
    } else {
      right <- compress(x, rson, depth + 1L)
      x <- right$x
    }

    maxd <- max(left$depth, right$depth) - depth
    mind <- min(left$depth, right$depth) - depth

    ## Find the smallest distance between the two subtrees
    ##   But only over depths that they have in common
    ## 1 is a minimum distance allowed
    slide <- min(right$left[1L:mind] - left$right[1L:mind]) - 1L
    if (slide > 0) { # slide the right hand node to the left
      x[right$sons] <- x[right$sons] - slide
      x[me] <- (x[right$sons[1L]] + x[left$sons[1L]]) / 2
    }
    else {
      slide <- 0
    }

    ## report back
    if (left$depth > right$depth) {
      templ <- left$left
      tempr <- left$right
      tempr[1L:mind] <- pmax(tempr[1L:mind], right$right - slide)
    } else {
      templ <- right$left - slide
      tempr <- right$right - slide
      templ[1L:mind] <- pmin(templ[1L:mind], left$left)
    }

    list(
      x = x,
      left = c(x[me] - nspace * (x[me] - x[lson]), templ),
      right = c(x[me] - nspace * (x[me] - x[rson]), tempr),
      depth = maxd + depth, sons = c(me, left$sons, right$sons)
    )
  }
  x <- compress(x, 1L, 1L)$x
  list(x = x, y = y)
}


# text.rpart ------------------------------------------------------------------

## This is a modification of text.tree.
## Fancy option has been added in (to mimic post.tree)
##

text_rpart <- function(x, splits = TRUE, label, FUN = text, all = FALSE,
                       pretty = NULL, digits = getOption("digits") - 3L,
                       use.n = FALSE, fancy = FALSE, fwidth = 0.8, fheight = 0.8,
                       bg = par("bg"), minlength = 1L, parms, ...) {
  string.bounding.box <- NULL
  if (!inherits(x, "rpart")) stop("Not a legitimate \"rpart\" object")
  if (nrow(x$frame) <= 1L) stop("fit is not a tree, just a root")

  frame <- x$frame
  if (!missing(label)) warning("argument 'label' is no longer used")
  col <- names(frame)
  ylevels <- attr(x, "ylevels")
  if (!is.null(ylevels <- attr(x, "ylevels"))) col <- c(col, ylevels)
  # cxy <- par("cxy")                   # character width and height
  cxy <- c(0.1, 0.1)
  if (!is.null(srt <- list(...)$srt) && srt == 90) cxy <- rev(cxy)
  xy <- rpartco(x, parms = parms)

  node <- as.numeric(row.names(frame))
  is.left <- (node %% 2L == 0L) # left hand sons
  node.left <- node[is.left]
  parent <- match(node.left / 2L, node)

  ## Put left splits at the parent node
  if (splits) {
    left.child <- match(2L * node, node)
    right.child <- match(node * 2L + 1L, node)
    rows <- if (!missing(pretty) && missing(minlength)) {
      labels(x, pretty = pretty)
    } else {
      labels(x, minlength = minlength)
    }
    if (fancy) {
      ## put split labels on branches instead of nodes
      xytmp <- rpart.branch(x = xy$x, y = xy$y, node = node)
      leftptx <- (xytmp$x[2L, ] + xytmp$x[1L, ]) / 2
      leftpty <- (xytmp$y[2L, ] + xytmp$y[1L, ]) / 2
      rightptx <- (xytmp$x[3L, ] + xytmp$x[4L, ]) / 2
      rightpty <- (xytmp$y[3L, ] + xytmp$y[4L, ]) / 2

      #         FUN(leftptx, leftpty + 0.52 * cxy[2L],
      #             rows[left.child[!is.na(left.child)]], ...)
      #         FUN(rightptx, rightpty - 0.52 * cxy[2L],
      #             rows[right.child[!is.na(right.child)]], ...)
    } else {
      #         FUN(xy$x, xy$y + 0.5 * cxy[2L], rows[left.child], ...)

      ### Insertion by Andrie de Vries to capture label splits
      labelSplits <- data.frame(
        x = xy$x,
        y = xy$y,
        label = rows[left.child]
      )
    }
    labelSplits <- labelSplits[complete.cases(labelSplits), ]
    labelSplits$type <= "splits"
  }

  leaves <- if (all) rep(TRUE, nrow(frame)) else frame$var == "<leaf>"

  stat <-
    x$functions$text(
      yval = if (is.null(frame$yval2)) {
        frame$yval[leaves]
      } else {
        frame$yval2[leaves, ]
      },
      dev = frame$dev[leaves], wt = frame$wt[leaves],
      ylevel = ylevels, digits = digits,
      n = frame$n[leaves], use.n = use.n
    )

  # if (fancy) {
  #   if (col2rgb(bg, alpha = TRUE)[4L, 1L] < 255) bg <- "white"
  #   oval <- function(middlex, middley, a, b)
  #   {
  #     theta <- seq(0, 2 * pi, pi/30)
  #     newx <- middlex + a * cos(theta)
  #     newy <- middley + b * sin(theta)
  #     polygon(newx, newy, border = TRUE, col = bg)
  #   }
  #
  #   ## FIXME: use rect()
  #   rectangle <- function(middlex, middley, a, b)
  #   {
  #     newx <- middlex + c(a, a, -a, -a)
  #     newy <- middley + c(b, -b, -b, b)
  #     polygon(newx, newy, border = TRUE, col = bg)
  #   }
  #
  #   ## find maximum length of stat
  #   maxlen <- max(string.bounding.box(stat)$columns) + 1L
  #   maxht <- max(string.bounding.box(stat)$rows) + 1L
  #
  #   a.length <- if (fwidth < 1)  fwidth * cxy[1L] * maxlen else fwidth * cxy[1L]
  #
  #   b.length <- if (fheight < 1) fheight * cxy[2L] * maxht else fheight * cxy[2L]
  #
  #   ## create ovals and rectangles here
  #   ## sqrt(2) creates the smallest oval that fits around the
  #   ## best fitting rectangle
  #   for (i in parent)
  #     oval(xy$x[i], xy$y[i], sqrt(2) * a.length/2, sqrt(2) * b.length/2)
  #   child <- match(node[frame$var == "<leaf>"], node)
  #   for (i in child)
  #     rectangle(xy$x[i], xy$y[i], a.length/2, b.length/2)
  # }

  ## if FUN=text then adj=1 puts the split label to the left of the
  ##    split rather than centered
  ## Allow labels at all or just leaf nodes

  ## stick values on nodes
  #     if (fancy) FUN(xy$x[leaves], xy$y[leaves] + 0.5 * cxy[2L], stat, ...)
  #     else FUN(xy$x[leaves], xy$y[leaves] - 0.5 * cxy[2L], stat, adj = 0.5, ...)
  labelLeaves <- data.frame(
    x = xy$x[leaves],
    y = xy$y[leaves],
    label = stat
  )



  #     return(list(xy=xy, cxy=cxy))
  list(
    labels = labelSplits,
    leaf_labels = labelLeaves
  )
}



# formatg -----------------------------------------------------------------



## format a set of numbers using C's "g" format
## No longer exported.
formatg <- function(x, digits = getOption("digits"),
                    format = paste0("%.", digits, "g")) {
  if (!is.numeric(x)) stop("'x' must be a numeric vector")

  temp <- sprintf(format, x)
  if (is.matrix(x)) matrix(temp, nrow = nrow(x)) else temp
}

#
#  ggdendro/R/dendro_tree.R by Andrie de Vries  Copyright (C) 2011-2015
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
#


#' Creates dendrogram plot using ggplot.
#'
#' This is a convenience function
#'
#' @param data Either a dendro object or an object that can be coerced to class
#'   dendro using the [dendro_data()] function, i.e. objects of class
#'   dendrogram, hclust or tree
#' 
#' @param segments If TRUE, show line segments
#' 
#' @param labels if TRUE, shows segment labels
#' 
#' @param leaf_labels if TRUE, shows leaf labels
#' 
#' @param rotate if TRUE, rotates plot by 90 degrees
#' 
#' @param theme_dendro if TRUE, applies a blank theme to plot (see
#'   [theme_dendro()])
#'   
#' @param ... other parameters passed to [ggplot2::geom_text()]
#' 
#' @export
#' @return A [ggplot2::ggplot()] object
#' @seealso [dendro_data()]
#' @example inst/examples/example_ggdendrogram.R
ggdendrogram <- function(data, 
                         segments = TRUE, labels = TRUE, leaf_labels = TRUE,
                         rotate = FALSE, theme_dendro = TRUE, ...) {
  dataClass <- if (inherits(data, "dendro")) data$class else class(data)
  angle <- if (dataClass %in% c("dendrogram", "hclust")) {
    ifelse(rotate, 0, 90)
  } else {
    ifelse(rotate, 90, 0)
  }
  hjust <- if (dataClass %in% c("dendrogram", "hclust")) {
    ifelse(rotate, 1, 1)
  } else {
    0.5
  }
  if (!is.dendro(data)) data <- dendro_data(data)
  p <- ggplot() +
    geom_blank()
  if (segments && !is.null(data$segments)) {
    p <- p + geom_segment(
      data = segment(data),
      aes(x = .data[["x"]], y = .data[["y"]], xend = .data[["xend"]], yend = .data[["yend"]])
    )
  }
  if (leaf_labels && !is.null(data$leaf_labels)) {
    p <- p + geom_text(
      data = leaf_label(data),
      aes(x = .data[["x"]], y = .data[["y"]], label = .data[["label"]]), hjust = hjust, angle = angle, ...
    )
  }
  if (labels) {
    p <- p + scale_x_continuous(
      breaks = seq_along(data$labels$label),
      labels = data$labels$label
    )
  }
  if (rotate) {
    p <- p + coord_flip()
    p <- p + scale_y_continuous()
  } else {
    p <- p + scale_y_continuous()
  }
  if (theme_dendro) p <- p + theme_dendro()
  p <- p +
    theme(axis.text.x = element_text(angle = angle, hjust = 1, vjust = 0.5)) +
    theme(axis.text.y = element_text(angle = angle, hjust = 1))

  p
}


#' Creates completely blank theme in ggplot.
#'
#' Sets most of the `ggplot` options to blank, by returning blank `theme`
#' elements for the panel grid, panel background, axis title, axis text, axis
#' line and axis ticks.
#' @export
theme_dendro <- function() {
  element_blank <- ggplot2::element_blank
  ggplot2::theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    axis.title.x = element_text(colour = NA),
    axis.title.y = element_blank(),
    axis.text.x = element_blank(),
    axis.text.y = element_blank(),
    axis.line = element_blank(),
    axis.ticks = element_blank()
  )
}

context("ggdendrogram")

test_that("ggdendrogram plots and accepts ... parameters", {
  
  require(ggplot2)  
  hc <- hclust(dist(USArrests), "ave")
  p1 <- ggdendrogram(hc)
  expect_that(p1, is_a("ggplot"))
  p2 <- ggdendrogram(hc, size=2)
  expect_that(p2, is_a("ggplot"))
  #print(p1)
  #print(p2)
  
})

# .memberDend -------------------------------------------------------------

### Code copied from stats:::.memberDend

.memberDend <- function(x) {
  r <- attr(x, "x.member")
  if (is.null(r)) {
    r <- attr(x, "members")
    if (is.null(r)) {
      r <- 1L
    }
  }
  r
}


# plotNodeLimit -----------------------------------------------------------

### Code copied from stats:::plotNodeLimit

plotNodeLimit <- function(x1, x2, subtree, center) {
  inner <- !is.leaf(subtree) && x1 != x2
  if (inner) {
    K <- length(subtree)
    mTop <- .memberDend(subtree)
    limit <- integer(K)
    xx1 <- x1
    for (k in 1L:K) {
      m <- .memberDend(subtree[[k]])
      xx1 <- xx1 + (if (center) {
        (x2 - x1) * m / mTop
      } else {
        m
      })
      limit[k] <- xx1
    }
    limit <- c(x1, limit)
  }
  else {
    limit <- c(x1, x2)
  }
  mid <- attr(subtree, "midpoint")
  center <- center || (inner && !is.numeric(mid))
  x <- if (center) {
    mean(c(x1, x2))
  } else {
    x1 + (if (inner) {
      mid
    } else {
      0
    })
  }
  list(x = x, limit = limit)
}

# .midDend --------------------------------------------------------------------

### Code copied from stats:::.midDend


.midDend <- function(x) {
  if (is.null(mp <- attr(x, "midpoint"))) 0 else mp
}

---
title: "Modifying ggdendogram output"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Modifying ggdendogram output}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


If you use `ggdendrogram()` to create your plot, the resulting object is a `ggplot`. You have full control over this using any function available in `ggplot`.

First create an example dataset.

```{r setup}
library(ggdendro)
library(ggplot2)
hc <- hclust(dist(USArrests), "ave")
```


Plot the default `ggdendrogram()` output:


```{r example-default}
ggdendrogram(hc, rotate = FALSE, size = 2)
```

Use a different theme:


```{r example-1}
ggdendrogram(hc, rotate = FALSE, size = 2) +
  theme_bw()
```

Or modify just one element, for example add a y-axis.


```{r example-2}
ggdendrogram(hc, rotate = FALSE, size = 2) +
  theme( axis.line.y = element_line() )
```

In summary, `ggdendrogram()` is a convenience function that creates a `ggplot`.  Once you have this plot, you can modify the plot using tools that you are familiar with.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_tree.R
\name{treeco}
\alias{treeco}
\title{Function copied from tree:::treeco.}
\usage{
treeco(tree, uniform)
}
\arguments{
\item{tree}{tree object}

\item{uniform}{???}
}
\description{
Function copied from tree:::treeco.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rpart.R
\name{rpartco}
\alias{rpartco}
\title{Compute the x-y coordinates for a tree.}
\usage{
rpartco(tree, parms)
}
\arguments{
\item{tree}{Tree model}

\item{parms}{Graphics parameters}
}
\description{
Compute the x-y coordinates for a tree.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_tree.R
\name{tree_segments}
\alias{tree_segments}
\title{Extract data frame from tree object for plotting using ggplot.}
\usage{
tree_segments(model, uniform, ...)
}
\arguments{
\item{model}{object of class "tree", e.g. the output of tree()}

\item{...}{ignored}
}
\description{
Extract data frame from tree object for plotting using ggplot.
}
\author{
Code modified from original by Brian Ripley
}
\seealso{
\code{\link{ggdendrogram}}

Other tree functions: \code{\link{dendro_data.tree}},
  \code{\link{get_data_tree_leaf_labels}},
  \code{\link{tree_labels}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdendrogram.R
\name{ggdendrogram}
\alias{ggdendrogram}
\title{Creates dendrogram plot using ggplot.}
\usage{
ggdendrogram(
  data,
  segments = TRUE,
  labels = TRUE,
  leaf_labels = TRUE,
  rotate = FALSE,
  theme_dendro = TRUE,
  ...
)
}
\arguments{
\item{data}{Either a dendro object or an object that can be coerced to class
dendro using the \code{\link[=dendro_data]{dendro_data()}} function, i.e. objects of class
dendrogram, hclust or tree}

\item{segments}{If TRUE, show line segments}

\item{labels}{if TRUE, shows segment labels}

\item{leaf_labels}{if TRUE, shows leaf labels}

\item{rotate}{if TRUE, rotates plot by 90 degrees}

\item{theme_dendro}{if TRUE, applies a blank theme to plot (see
\code{\link[=theme_dendro]{theme_dendro()}})}

\item{...}{other parameters passed to \code{\link[ggplot2:geom_text]{ggplot2::geom_text()}}}
}
\value{
A \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} object
}
\description{
This is a convenience function
}
\examples{
### Demonstrate ggdendrogram

library(ggplot2)
hc <- hclust(dist(USArrests), "ave")

# Demonstrate plotting directly from object class hclust
p <- ggdendrogram(hc, rotate = FALSE)
print(p)
ggdendrogram(hc, rotate = TRUE)

# demonstrate converting hclust to dendro using dendro_data first
hcdata <- dendro_data(hc)
ggdendrogram(hcdata, rotate = TRUE, size = 2) + 
  labs(title = "Dendrogram in ggplot2")
}
\seealso{
\code{\link[=dendro_data]{dendro_data()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_data.R
\name{segment}
\alias{segment}
\alias{label}
\alias{leaf_label}
\title{Returns segment, label or leaf-label data from dendro object.}
\usage{
segment(x)

label(x)

leaf_label(x)
}
\arguments{
\item{x}{dendro object}
}
\description{
\code{segment} extracts line segments, \code{label} extracts labels, and \code{leaf_label}
extracts leaf labels from a dendro object.
}
\seealso{
\code{\link[=dendro_data]{dendro_data()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_rpart.R
\name{rpart_labels}
\alias{rpart_labels}
\title{Extract labels data frame from rpart object for plotting using ggplot.}
\usage{
rpart_labels(x)
}
\value{
a list with two elements: $labels and $leaf_labels
}
\description{
This code is modified from the original plot.rpart in package rpart.
}
\seealso{
\code{\link[=ggdendrogram]{ggdendrogram()}}

Other dendro_data methods: 
\code{\link{dendro_data}()},
\code{\link{dendro_data.rpart}()},
\code{\link{dendro_data.tree}()},
\code{\link{dendrogram_data}()}

Other rpart functions: 
\code{\link{dendro_data.rpart}()},
\code{\link{rpart_segments}()}
}
\author{
Original author Brian Ripley
}
\concept{dendro_data methods}
\concept{rpart functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdendro-package.R
\docType{package}
\name{ggdendro-package}
\alias{ggdendro}
\alias{ggdendro-package}
\title{Create Dendrograms and Tree Diagrams using 'ggplot2'}
\description{
This package enables you to create dendrograms and tree plots using
\code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\details{
The \code{ggplot2} philosophy is to clearly separate data from the presentation.
Unfortunately the plot method for dendrograms (\code{\link[=plot.dendrogram]{plot.dendrogram()}}) plots
directly to a plot device without exposing the data.  The \code{ggdendro} package
resolves this by making available functions that extract the dendrogram plot
data.  This data can be used with \code{ggplot}.

The function \code{\link[=dendro_data]{dendro_data()}} extracts data from different objects that
contain dendrogram information.  It is a generic function with methods for:
\itemize{
\item \code{hclust}: \code{\link[=dendro_data.hclust]{dendro_data.hclust()}}
\item dendrogram: \code{\link[=dendro_data.dendrogram]{dendro_data.dendrogram()}}
\item regression trees: \code{\link[=dendro_data.tree]{dendro_data.tree()}}
\item partition trees: \code{\link[=dendro_data.rpart]{dendro_data.rpart()}}
\item \code{agnes} and \code{diana}: \code{\link[=dendro_data.twins]{dendro_data.twins()}}
}

These methods create an object of class \code{dendro}, consisting of a list of
data frames.  To extract the relevant data frames from the list, you can use
the accessor functions:
\itemize{
\item \code{\link[=segment]{segment()}}: the line segment data
\item \code{\link[=label]{label()}}: the text for each end segment
\item \code{\link[=leaf_label]{leaf_label()}}: the leaf labels of a tree diagram
}

To plot a dendrogram, either construct a plot with \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} or use
the function \code{\link[=ggdendrogram]{ggdendrogram()}}.
}
\seealso{
\code{\link[=dendro_data]{dendro_data()}}
}
\author{
Andrie de Vries - \email{apdevries@gmail.com}
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_tree.R
\name{get_data_tree_leaf_labels}
\alias{get_data_tree_leaf_labels}
\title{Extract labels data frame from tree object for plotting using ggplot.}
\usage{
get_data_tree_leaf_labels(model, uniform, ...)
}
\arguments{
\item{model}{object of class "tree", e.g. the output of tree()}

\item{...}{ignored}
}
\description{
Extract labels data frame from tree object for plotting using ggplot
}
\seealso{
\code{\link[=ggdendrogram]{ggdendrogram()}}

Other tree functions: 
\code{\link{dendro_data.tree}()},
\code{\link{tree_labels}()},
\code{\link{tree_segments}()}
}
\author{
Code modified from original by Brian Ripley
}
\concept{tree functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendrogram.R
\name{dendrogram_data}
\alias{dendrogram_data}
\title{Extract data frame from dendrogram object for plotting using ggplot.}
\usage{
dendrogram_data(x, type = c("rectangle", "triangle"), ...)
}
\arguments{
\item{x}{object of class "dendrogram", e.g. the output of as.dendrogram()}

\item{type}{The type of plot, indicating the shape of the dendrogram:
"rectangle" will draw rectangular lines, while "triangle" will draw
triangular lines.}

\item{...}{ignored}
}
\description{
Extract data frame from dendrogram object for plotting using ggplot
}
\seealso{
\code{\link[=ggdendrogram]{ggdendrogram()}}

Other dendro_data methods: 
\code{\link{dendro_data}()},
\code{\link{dendro_data.rpart}()},
\code{\link{dendro_data.tree}()},
\code{\link{rpart_labels}()}

Other dendrogram/hclust functions: 
\code{\link{dendro_data}()}
}
\concept{dendro_data methods}
\concept{dendrogram/hclust functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_data.R, R/dendrogram.R
\name{dendro_data}
\alias{dendro_data}
\alias{dendro_data.default}
\alias{dendro_data.dendrogram}
\alias{dendro_data.hclust}
\alias{dendro_data.twins}
\title{Extract cluster data from a model into a list of data frames.}
\usage{
dendro_data(model, ...)

\method{dendro_data}{default}(model, ...)

\method{dendro_data}{dendrogram}(model, type = c("rectangle", "triangle"), ...)

\method{dendro_data}{hclust}(model, type = c("rectangle", "triangle"), ...)

\method{dendro_data}{twins}(model, type = c("rectangle", "triangle"), ...)
}
\arguments{
\item{model}{object of class "dendrogram", e.g. the output of as.dendrogram()}

\item{...}{ignored}

\item{type}{The type of plot, indicating the shape of the dendrogram.
"rectangle" will draw rectangular lines, while "triangle" will draw
triangular lines.}
}
\value{
a list of data frames that contain the data appropriate to each
cluster model

A list with components:
\item{segments}{Line segment data}
\item{labels}{Label data}
}
\description{
This function provides a generic mechanism to extract relevant plotting data,
typically line segments and labels, from a variety of cluster models.

Extract line segment and label data from \code{\link[stats:dendrogram]{stats::dendrogram()}} or
\code{\link[stats:hclust]{stats::hclust()}} object.  The resulting object is a list of data frames
containing line segment data and label data.
}
\details{
For \code{\link[stats:dendrogram]{stats::dendrogram()}} and \code{tree::tree()} models, extracts line segment
data and labels.
}
\examples{
require(ggplot2)

### Demonstrate dendro_data.dendrogram

model <- hclust(dist(USArrests), "ave")
dendro <- as.dendrogram(model)

# Rectangular lines
ddata <- dendro_data(dendro, type = "rectangle")
ggplot(segment(ddata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  coord_flip() +
  scale_y_reverse(expand = c(0.2, 0)) +
  theme_dendro()

# Triangular lines
ddata <- dendro_data(dendro, type = "triangle")
ggplot(segment(ddata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  theme_dendro()

# Demonstrate dendro_data.hclust

require(ggplot2)
hc <- hclust(dist(USArrests), "ave")

# Rectangular lines
hcdata <- dendro_data(hc, type = "rectangle")
ggplot(segment(hcdata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  coord_flip() +
  scale_y_reverse(expand = c(0.2, 0)) +
  theme_dendro()

# Triangular lines
hcdata <- dendro_data(hc, type = "triangle")
ggplot(segment(hcdata)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  theme_dendro()
### Demonstrate the twins of agnes and diana, from package cluster

if (require(cluster)) {
  model <- agnes(votes.repub, metric = "manhattan", stand = TRUE)
  dg <- as.dendrogram(model)
  ggdendrogram(dg)
}


if (require(cluster)) {
  model <- diana(votes.repub, metric = "manhattan", stand = TRUE)
  dg <- as.dendrogram(model)
  ggdendrogram(dg)
}
}
\seealso{
There are several implementations for specific cluster algorithms:
\itemize{
\item \code{\link[=dendro_data.hclust]{dendro_data.hclust()}}
\item \code{\link[=dendro_data.dendrogram]{dendro_data.dendrogram()}}
\item \code{\link[=dendro_data.tree]{dendro_data.tree()}}
\item \code{\link[=dendro_data.rpart]{dendro_data.rpart()}}
}

To extract the data for line segments, labels or leaf labels use:
\itemize{
\item \code{\link[=segment]{segment()}}: the line segment data
\item \code{\link[=label]{label()}}: the text for each end segment
\item \code{\link[=leaf_label]{leaf_label()}}: the leaf labels of a tree diagram
}

\code{\link[=ggdendrogram]{ggdendrogram()}}

Other dendro_data methods: 
\code{\link{dendro_data.rpart}()},
\code{\link{dendro_data.tree}()},
\code{\link{dendrogram_data}()},
\code{\link{rpart_labels}()}

Other dendrogram/hclust functions: 
\code{\link{dendrogram_data}()}
}
\concept{dendro_data methods}
\concept{dendrogram/hclust functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_data.R
\name{as.dendro}
\alias{as.dendro}
\title{Coerces object to class dendro.}
\usage{
as.dendro(segments, labels, leaf_labels = NULL, class)
}
\arguments{
\item{segments}{data.frame with segment data}

\item{labels}{data.frame with labels data}

\item{leaf_labels}{data.frame with leaf label data}

\item{class}{The class of the original model object, e.g. "hclust".  This is
used by \code{\link[=ggdendrogram]{ggdendrogram()}} to determine the angle and justification
of labels}
}
\description{
Method for coercing object to class dendro.
}
\seealso{
\code{\link[=dendro_data]{dendro_data()}} and \code{\link[=ggdendro-package]{ggdendro-package()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_data.R
\name{is.dendro}
\alias{is.dendro}
\title{Tests whether an object is of class dendro.}
\usage{
is.dendro(x)
}
\arguments{
\item{x}{Object to check}
}
\description{
Is a dendro?  Tests whether an object is of class dendro.
}
\seealso{
\code{\link[=dendro_data]{dendro_data()}} and \code{\link[=ggdendro-package]{ggdendro-package()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_tree.R
\name{dendro_data.tree}
\alias{dendro_data.tree}
\title{Extract data from regression tree object for plotting using ggplot.}
\usage{
\method{dendro_data}{tree}(model, type = c("proportional", "uniform"), ...)
}
\arguments{
\item{model}{object of class "tree", e.g. the output of tree()}

\item{type}{Either \code{proportional} or \code{uniform}. If this partially matches
"uniform", the branches are of uniform length. Otherwise they are
proportional to the decrease in impurity.}

\item{...}{ignored}
}
\value{
A list of three data frames:
\item{segments}{a data frame containing the line segment data}
\item{labels}{a data frame containing the label text data}
\item{leaf_labels}{a data frame containing the leaf label text data}
}
\description{
Extracts data to plot line segments and labels from a \code{tree::tree()} object.
This data can then be manipulated or plotted, e.g. using \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\examples{
### Demonstrate tree

if (require(tree)) {
  require(ggplot2)
  require(MASS)
  data(cpus, package = "MASS")
  cpus.ltr <- tree(log10(perf) ~ syct + mmin + mmax + cach + chmin + chmax, 
                   data = cpus)
  tree_data <- dendro_data(cpus.ltr)
  ggplot(segment(tree_data)) +
    geom_segment(aes(x = x, y = y, xend = xend, yend = yend, linewidth = n),
      colour = "lightblue"
    ) +
    scale_size("n") +
    geom_text(
      data = label(tree_data),
      aes(x = x, y = y, label = label), vjust = -0.5, size = 4
    ) +
    geom_text(
      data = leaf_label(tree_data),
      aes(x = x, y = y, label = label), vjust = 0.5, size = 3
    ) +
    theme_dendro()
}
}
\seealso{
\code{\link[=ggdendrogram]{ggdendrogram()}}

Other dendro_data methods: 
\code{\link{dendro_data}()},
\code{\link{dendro_data.rpart}()},
\code{\link{dendrogram_data}()},
\code{\link{rpart_labels}()}

Other tree functions: 
\code{\link{get_data_tree_leaf_labels}()},
\code{\link{tree_labels}()},
\code{\link{tree_segments}()}
}
\author{
Andrie de Vries, using code modified from original by Brian Ripley
}
\concept{dendro_data methods}
\concept{tree functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_rpart.R
\name{rpart_segments}
\alias{rpart_segments}
\title{Extract data frame from rpart object for plotting using ggplot.}
\usage{
rpart_segments(x, ...)
}
\arguments{
\item{...}{ignored}
}
\description{
Extract data frame from rpart object for plotting using ggplot.
}
\seealso{
\code{\link[=ggdendrogram]{ggdendrogram()}}

Other rpart functions: 
\code{\link{dendro_data.rpart}()},
\code{\link{rpart_labels}()}
}
\concept{rpart functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rpart.R
\name{labels.rpart}
\alias{labels.rpart}
\title{Make the nice labels used by print and summary.}
\usage{
\method{labels}{rpart}(object, digits = 4, minlength = 1L, pretty, collapse = TRUE, ...)
}
\arguments{
\item{digits}{obvious}

\item{minlength}{0 = don't abbrev factors; 1 = use single letters; 2+= the
same arg as the "abbreviate" function}

\item{collapse}{an oddly named argument: FALSE = return a matrix with two
columns, containing the labels of the left and right descendants of each
node; TRUE = return a vector of 1 column, with the label of the parent}

\item{...}{= other args for abbreviate()}

\item{pretty:}{for historical compatibility: 0   -> minlength = 0; NULL ->
minlength = 1; TRUE   -> minlength = 4}
}
\description{
Make the nice labels used by print and summary.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdendrogram.R
\name{theme_dendro}
\alias{theme_dendro}
\title{Creates completely blank theme in ggplot.}
\usage{
theme_dendro()
}
\description{
Sets most of the \code{ggplot} options to blank, by returning blank \code{theme}
elements for the panel grid, panel background, axis title, axis text, axis
line and axis ticks.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro_rpart.R
\name{dendro_data.rpart}
\alias{dendro_data.rpart}
\title{Extract data from classification tree object for plotting using ggplot.}
\usage{
\method{dendro_data}{rpart}(
  model,
  uniform = FALSE,
  branch = 1,
  compress = FALSE,
  nspace,
  minbranch = 0.3,
  ...
)
}
\arguments{
\item{model}{object of class "tree", e.g. the output of tree()}

\item{uniform}{if TRUE, uniform vertical spacing of the nodes is used; this
may be less cluttered when fitting a large plot onto a page. The default is
to use a non-uniform spacing proportional to the error in the fit.}

\item{branch}{controls the shape of the branches from parent to child node.
Any number from 0 to 1 is allowed. A value of 1 gives square shouldered
branches, a value of 0 give V shaped branches, with other values being
intermediate.}

\item{compress}{if FALSE, the leaf nodes will be at the horizontal plot
coordinates of 1:nleaves. If TRUE, the routine attempts a more compact
arrangement of the tree. The compaction algorithm assumes uniform=TRUE;
surprisingly, the result is usually an improvement even when that is not
the case.}

\item{nspace}{the amount of extra space between a node with children and a
leaf, as compared to the minimal space between leaves. Applies to
compressed trees only. The default is the value of branch.}

\item{minbranch}{set the minimum length for a branch to minbranch times the
average branch length. This parameter is ignored if uniform=TRUE. Sometimes
a split will give very little improvement, or even (in the classification
case) no improvement at all. A tree with branch lengths strictly
proportional to improvement leaves no room to squeeze in node labels.}

\item{...}{ignored}
}
\value{
A list of three data frames:
\item{segments}{a data frame containing the line segment data}
\item{labels}{a data frame containing the label text data}
\item{leaf_labels}{a data frame containing the leaf label text data}
}
\description{
Extracts data to plot line segments and labels from a
\code{\link[rpart:rpart]{rpart::rpart()}} classification tree object.  This data can then be
manipulated or plotted, e.g. using \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\details{
This code is in essence a copy of \code{\link[rpart:plot.rpart]{rpart::plot.rpart()}}, retaining
the plot data but without plotting to a plot device.
}
\examples{
### Demonstrate rpart

if (require(rpart)) {
  require(ggplot2)
  fit <- rpart(Kyphosis ~ Age + Number + Start, method = "class", 
               data = kyphosis)
  fitr <- dendro_data(fit)
  ggplot() +
    geom_segment(data = fitr$segments, 
                 aes(x = x, y = y, xend = xend, yend = yend)
    ) +
    geom_text(data = fitr$labels, aes(x = x, y = y, label = label)) +
    geom_text(data = fitr$leaf_labels, aes(x = x, y = y, label = label)) +
    theme_dendro()
}
}
\seealso{
\code{\link[=ggdendrogram]{ggdendrogram()}}

Other dendro_data methods: 
\code{\link{dendro_data}()},
\code{\link{dendro_data.tree}()},
\code{\link{dendrogram_data}()},
\code{\link{rpart_labels}()}

Other rpart functions: 
\code{\link{rpart_labels}()},
\code{\link{rpart_segments}()}
}
\concept{dendro_data methods}
\concept{rpart functions}
